package org.apache.bcel.verifier.structurals;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import org.apache.bcel.generic.InstructionHandle;
import org.apache.bcel.generic.MethodGen;
import org.apache.bcel.verifier.exc.AssertionViolatedException;
import org.apache.bcel.verifier.structurals.ExceptionHandlers;
import org.apache.bcel.verifier.structurals.InstructionContext;
import org.apache.bcel.verifier.structurals.Subroutines;

public class ControlFlowGraph {
  private final Subroutines subroutines;
  
  private final ExceptionHandlers exceptionhandlers;
  
  private final Map<InstructionHandle, InstructionContext> instructionContexts = new HashMap<>();
  
  public ControlFlowGraph(MethodGen methodGen) {
    this(methodGen, true);
  }
  
  public ControlFlowGraph(MethodGen methodGen, boolean enableJustIceCheck) {
    this.subroutines = new Subroutines(methodGen, enableJustIceCheck);
    this.exceptionhandlers = new ExceptionHandlers(methodGen);
    InstructionHandle[] instructionhandles = methodGen.getInstructionList().getInstructionHandles();
    for (InstructionHandle instructionhandle : instructionhandles)
      this.instructionContexts.put(instructionhandle, new InstructionContextImpl(this, instructionhandle)); 
  }
  
  public InstructionContext contextOf(InstructionHandle inst) {
    InstructionContext ic = this.instructionContexts.get(inst);
    if (ic == null)
      throw new AssertionViolatedException("InstructionContext requested for an InstructionHandle that's not known!"); 
    return ic;
  }
  
  public InstructionContext[] contextsOf(InstructionHandle[] insts) {
    InstructionContext[] ret = new InstructionContext[insts.length];
    Arrays.setAll(ret, i -> contextOf(insts[i]));
    return ret;
  }
  
  public InstructionContext[] getInstructionContexts() {
    InstructionContext[] ret = new InstructionContext[this.instructionContexts.size()];
    return (InstructionContext[])this.instructionContexts.values().toArray((Object[])ret);
  }
  
  public boolean isDead(InstructionHandle i) {
    return (this.subroutines.subroutineOf(i) == null);
  }
}
